@@ -1,4 +1,8 @@
+# Copyright (c) Jupyter Development Team.
+# Distributed under the terms of the Modified BSD License.
+
 """Test the kernels service API."""
+
 import asyncio
 import json
 import os
@@ -104,12 +108,11 @@ class TestBuildAPI:
         assert expected_http_error(e, 500)
 
         loop = asyncio.get_event_loop()
-        asyncio.ensure_future(build_api_tester.build(), loop=loop)
+        asyncio.ensure_future(build_api_tester.build(), loop=loop)  # noqa RUF006
 
         while True:
             r = await build_api_tester.getStatus()
             res = r.body.decode()
-            print(res)
             resp = json.loads(res)
             if resp["status"] == "building":
                 break
